# -*- coding: utf8 -*-
 
# Ce script récupère le flux d'une webcam et le traite de manière à pouvoir
# l'envoyer en bitmap (1 bit = 1 pixel) par UDP sur un serveur
 
# Librairie OpenCV de traitement d'image.
# Non standard, donc à installer :
# sudo apt-get install python-opencv
import cv2
 
# Va nous permettre de faire de l'UDP.
import socket
 
# Va nous permettre de lire l'argument donné au lancement
import sys

from affichage import affichage
 
la_bas = sys.argv[1]
 
DICO_IP = {
"Nantes":"11.0.0.2",
"Toulouse":"11.0.0.3"
}
 
# IP et port de notre serveur.
UDP_IP_LA_BAS = DICO_IP[la_bas]
UDP_PORT = 5005
 
# On choisi /dev/video0 comme périphérique.
cap = cv2.VideoCapture(0)
 
# 160x120 étant la plus petite définition que l'on puisse capturer avec notre
# webcam, on définit la capture avec ces valeurs.
 
# Largeur.
cap.set(3, 160)
# Hauteur.
cap.set(4, 120)
 
# On définit les dimensions finales.
# Cela correspond au nombre de caractères affichables par défaut
# dans la console d'un RaspberryPi (lowtech powa).
larg_minitel = 71
haut_minitel = 31
 
# n° de la permier image (nécessaire si sauvegarde)
#num_image = 0

while True:
 
    # Lit la frame.
    ret, frame = cap.read()
 
    # Retaille la frame aux dimensions voulues.
    image_retaillee = cv2.resize(frame, (larg_minitel, haut_minitel))
 
    # Passe la frame en niveau de gris (une seule valeure pour chaque pixel)
    image_grise = cv2.cvtColor(image_retaillee,cv2.COLOR_BGR2GRAY)
 
    # Passe la frame en bitmap (0 ou 255 pour chaque pixel)
    #ret, image_bitmap = cv2.threshold(image_retaillee, 127, 255, cv2.THRESH_BINARY)
 
    # Nous avons fait le choix d'utiliser des images en bitmap.
    # C'est à dire que les pixels sont, soit noirs, soit blancs.
    # Ils sont donc transmis sous forme binaire (1 bit = 1 pixel).
    # Pour cela nous avons besoin de les stocker par octet dans un tableau.
    tab_octet = []
 
    # Premier octet vide.
    octet = ''
 
    # Initialisation des n° de pixel
    num_pixel = 0
 
    image_01 = ""    
 
    # Pour chaque ligne de l'image.
    for ligne in range(haut_minitel):    
 
        # Pour chaque pixel que chaque ligne.
        for pixel in range(larg_minitel):
 
            # On extrait la valeur de niveau de gris
            # de chaque pixel (0-255).
            val_coul = image_grise.item(ligne, pixel)
 
            # En fonction d'un seuil,
            if val_coul > 100:
 
                # le pixel est stocké sous la forme d'un 1,
                # dans un "octet" pour l'envoi
                octet += "1"
                # dans un fichier général pour l'affichage
                #image_01 += "1"
 
            else:
 
                # il est stocké sous la forme d'un 0.
                # dans un "octet" pour l'envoi
                octet += "0"
                # dans un fichier général pour l'affichage
                #image_01 += "0"
 
            # On incrémente le n° du pixel traité,
            # pour pouvoir les compter par paquet de 8.
            num_pixel += 1
 
            # Quand un octet est complet,
            if (num_pixel%8 == 0):
 
                # il est converti sous forme décimale,
                # puis stocké dans un tableau.
                tab_octet.append(int(octet, 2))
                # On repart alors sur un octet vide.
                octet = ''
 
    # Affiche l'image.
    #cv2.imshow('video test',image_grise)
 
    # Les octets, stockés jusque là dans un tableau sous forme décimale,
    # sont convertis sous forme binaire,
    data = bytearray(tab_octet)
 
    # puis envoyés en UDP sur un serveur.
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) 
    sock.sendto(data, (UDP_IP_LA_BAS, UDP_PORT))
 
    # Petite sauvegarde en local au besoin.
    #with open ("./img/{0}".format(num_image), 'w') as base:
    #    base.write(data)
 
    # On incrémente le n° de l'image.
    #num_image += 1
 
    #affichage(image_01)
    # On attend 40 ms pour obtenir du 25 fps
    key = cv2.waitKey(80)
